# -*- coding: utf-8 -*-

"""
    colorful
    ~~~~~~~~

    Terminal string styling done right, in Python.

    :copyright: (c) 2017 by Timo Furrer <tuxtimo@gmail.com>
    :license: MIT, see LICENSE for more details.
"""


#: Holds the solarized style
SOLARIZED = {
    'base03': '#002b36',
    'base02': '#073642',
    'base01': '#586e75',
    'base00': '#657b83',
    'base0': '#839496',
    'base1': '#93a1a1',
    'base2': '#eee8d5',
    'base3': '#fdf6e3',
    'yellow': '#b58900',
    'orange': '#cb4b16',
    'red': '#dc322f',
    'magenta': '#d33682',
    'violet': '#6c71c4',
    'blue': '#268bd2',
    'cyan': '#2aa198',
    'green': '#859900'
}

#: Holds the monokai style
MONOKAI = {
    'ghostWhite': '#F8F8F0',
    'lightGhostWhite': '#F8F8F2',
    'lightGray': '#CCCCCC',
    'gray': '#888888',
    'brownGray': '#49483E',
    'darkGray': '#282828',
    'yellow': '#E6DB74',
    'blue': '#66D9EF',
    'magenta': '#F92672',
    'purple': '#AE81FF',
    'brown': '#75715E',
    'orange': '#FD971F',
    'lightOrange': '#FFD569',
    'green': '#A6E22E',
    'seaGreen': '#529B2F'
}
